@@ -1,10 +1,7 @@
 #include <config.h>
 #include <libgnome/libgnome.h>
-#define USE_WNCK
-#ifdef USE_WNCK
-  #define WNCK_I_KNOW_THIS_IS_UNSTABLE 
-  #include <libwnck/libwnck.h>
-#endif
+#define WNCK_I_KNOW_THIS_IS_UNSTABLE 
+#include <libwnck/libwnck.h>
 #include <dirent.h>
 #include <sys/stat.h>
 #include <stdio.h>
@@ -16,11 +13,10 @@
 void free_entry (gpointer key, gpointer value, gpointer data);
 void free_value (gpointer key, gpointer value, gpointer data);
 void free_key (gpointer key, gpointer value, gpointer data);
-#ifdef USE_WNCK
-  WnckScreen *screen = NULL;
-#endif
 
-#ifdef USE_WNCK
+WnckScreen *screen = NULL;
+
+
 static void
 new_application (WnckScreen *screen, WnckApplication *app, gpointer data)
 {
@@ -28,27 +24,40 @@ new_application (WnckScreen *screen, WnckApplication *app, gpointer data)
 	ProcInfo *info;
 	GHashTable *hash = procdata->pretty_table->app_hash;
 	gint pid;
-	GdkPixbuf *icon = NULL;
+	GdkPixbuf *icon = NULL, *tmp = NULL;
 	GList *list = NULL;
 	WnckWindow *window;
+	gchar *text;
 	
 	pid = wnck_application_get_pid (app);
 	if (pid == 0)
 		return;
 	
+	/* Check to see if the pid has already been added */	
+	text = g_strdup_printf ("%d", pid);
+	if (g_hash_table_lookup (hash, text)) {
+		g_free (text);
+		return;
+	}
+	g_free (text);
+		
 	/* Hack - wnck_application_get_icon always returns the default icon */
 #if 0	
-	icon = wnck_application_get_mini_icon (app);	
+	tmp = wnck_application_get_mini_icon (app);	
 #else
 	list = wnck_application_get_windows (app);
 	if (!list)
 		return;
 	window = list->data;
-	icon = wnck_window_get_icon (window);
+	tmp = wnck_window_get_icon (window);
 #endif
+	if (!tmp)
+		return;
+		
+	icon =  gdk_pixbuf_scale_simple (tmp, 16, 16, GDK_INTERP_HYPER);
 	if (!icon)
 		return;
-	
+
 	/* If process already exists then set the icon. Otherwise put into hash
 	** table to be added later */	
 	info = proctable_find_process (pid, NULL, procdata);
@@ -57,16 +66,15 @@ new_application (WnckScreen *screen, WnckApplication *app, gpointer data)
 		/* Don't update the icon if there already is one */
 		if (info->pixbuf)
 			return;
-		info->pixbuf = gdk_pixbuf_scale_simple (icon, 16, 16, 
-							GDK_INTERP_HYPER);		
+		info->pixbuf = icon;		
 		if (info->visible) {
 			GtkTreeModel *model;
 			model = gtk_tree_view_get_model (GTK_TREE_VIEW (procdata->tree));
 			gtk_tree_store_set (GTK_TREE_STORE (model), &info->node,
-                            		    COL_PIXBUF, info->pixbuf, -1);
+                            		            COL_PIXBUF, info->pixbuf, -1);
                 }
 	}
-							
+						
 	g_hash_table_insert (hash, g_strdup_printf ("%d", pid), icon);
 		
 	
@@ -87,22 +95,26 @@ application_finished (WnckScreen *screen, WnckApplication *app, gpointer data)
 	pid =  wnck_application_get_pid (app);
 	if (pid == 0)
 		return;
-		
+	
 	id = g_strdup_printf ("%d", pid);
 	if (g_hash_table_lookup_extended (hash, id, &p1, &p2)) {
 		orig_id = p1;
+		icon = p2;
 		g_hash_table_remove (hash, orig_id);
 		if (orig_id) 
 			g_free (orig_id);
+		if (icon)
+			g_object_unref (G_OBJECT (icon));
 	}
 	
 	
 	g_free (id);
+
 	
 }
-#endif
 
-PrettyTable *pretty_table_new (ProcData *procdata) {
+PrettyTable *pretty_table_new (ProcData *procdata) 
+{
 	PrettyTable *pretty_table = NULL;
 	GList *list = NULL;
 	
@@ -110,49 +122,21 @@ PrettyTable *pretty_table_new (ProcData *procdata) {
 	
 	pretty_table->app_hash = g_hash_table_new (g_str_hash, g_str_equal);
 	pretty_table->default_hash = g_hash_table_new (g_str_hash, g_str_equal);
-	
-#ifdef USE_WNCK
-	screen = wnck_screen_get_default ();
-	wnck_screen_force_update (screen);
-	list = wnck_screen_get_windows (screen);
-	while (list) {
-		WnckWindow *window = list->data;
-		gint pid;
-		
-		pid = wnck_window_get_pid (window);
-		if (pid > 0) {
-			GdkPixbuf *icon;
-			gchar *id;
-			icon = wnck_window_get_icon (window);
-			if (icon) {
-				id = g_strdup_printf ("%d", pid);
-				/* Don't put in icons for the same pid */
-				if (!g_hash_table_lookup (pretty_table->app_hash, id)) 
-				 	g_hash_table_insert (pretty_table->app_hash, 
-						             id, icon);
-					
-				else 
-					g_free (id);
-			}
-		}
-		
-		list = g_list_next (list);
-	}	
-#endif	
 
-	
-#ifdef USE_WNCK	
+	screen = wnck_screen_get_default ();
+		
 	g_signal_connect (G_OBJECT (screen), "application_opened",
 			  G_CALLBACK (new_application), procdata);
 	g_signal_connect (G_OBJECT (screen), "application_closed",
 			  G_CALLBACK (application_finished), procdata);
-#endif
+
 	pretty_table_add_table (pretty_table, default_table);
 	
 	return pretty_table;
 }
 
-void pretty_table_add_table (PrettyTable *pretty_table, const gchar *table[]) {
+void pretty_table_add_table (PrettyTable *pretty_table, const gchar *table[]) 
+{
 	/* Table format:
 
 	const gchar *table[] = {
@@ -171,10 +155,22 @@ void pretty_table_add_table (PrettyTable *pretty_table, const gchar *table[]) {
 		text = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_DATADIR, "pixmaps/",
 						  TRUE, NULL);
 		if (text) {
+			GdkPixbuf *tmp = NULL, *icon = NULL;
 			prettyicon = g_malloc (strlen (table[i + 2]) + strlen (text) + 1);
 			sprintf (prettyicon, "%s%s", text, table[i + 2]);
-			g_hash_table_insert (pretty_table->default_hash, 
-					     command, prettyicon);
+			if (prettyicon) {
+				tmp = gdk_pixbuf_new_from_file (prettyicon, NULL);
+				g_free (prettyicon);		
+			}
+			if (tmp) {
+				icon =  gdk_pixbuf_scale_simple (tmp, 16, 16, 
+										 GDK_INTERP_HYPER);
+				g_object_unref (G_OBJECT (tmp));
+			}
+			if (icon) {
+				g_hash_table_insert (pretty_table->default_hash, 
+					                       command, icon);
+			};
 			g_free (text);
 		}
 		i += 3;
@@ -183,25 +179,6 @@ void pretty_table_add_table (PrettyTable *pretty_table, const gchar *table[]) {
 	return;
 }
 
-#if 0
-gchar *pretty_table_get_name (PrettyTable *pretty_table, const gchar *command) {
-	gchar *pretty_name;
-
-	if (!pretty_table)
-		return NULL;
-		
-	pretty_name = g_hash_table_lookup (pretty_table->cmdline_to_prettyname, command);
-	if (pretty_name) 
-		return g_strdup (pretty_name); 
-
-	pretty_name = g_hash_table_lookup (pretty_table->name_to_prettyname, command);
-	if (pretty_name) 
-		return g_strdup (pretty_name);
-
-	return NULL;
-}
-#endif
-
 GdkPixbuf *pretty_table_get_icon (PrettyTable *pretty_table, gchar *command, gint pid) 
 {
 	GdkPixbuf *icon = NULL, *tmp_pixbuf = NULL;
@@ -211,29 +188,15 @@ GdkPixbuf *pretty_table_get_icon (PrettyTable *pretty_table, gchar *command, gin
 	if (!pretty_table) 
 		return NULL;
 
-	icon_path = g_hash_table_lookup (pretty_table->default_hash, command);
-	
-	if (icon_path) {
-		tmp_pixbuf = gdk_pixbuf_new_from_file (icon_path, NULL);		
-	}
-	/* This looks ugly but we don't want to unref the pixbuf in the 
-	** app hash since it's owned by libwnck and not us */	
-	if (tmp_pixbuf) {
-		icon = gdk_pixbuf_scale_simple (tmp_pixbuf, 16, 16, 
-						GDK_INTERP_HYPER);
-		g_object_unref (tmp_pixbuf);
+
+	icon = g_hash_table_lookup (pretty_table->default_hash, command);
+	if (icon)
 		return icon;
-	}
-		
+	
 	text = g_strdup_printf ("%d", pid);
-	tmp_pixbuf = g_hash_table_lookup (pretty_table->app_hash, text);
+	icon = g_hash_table_lookup (pretty_table->app_hash, text);
 	g_free (text);
 	
-	if (!tmp_pixbuf) 
-		return NULL;
-	
-	icon = gdk_pixbuf_scale_simple (tmp_pixbuf, 16, 16, GDK_INTERP_HYPER);
-		
 	return icon;
 	
 }
